//
//  ViewController.swift
//  GPS
//
//  Created by Phidgets on 2018-01-30.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    
    let ch = GPS()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    
    @IBOutlet var fixAquiredLabel: UILabel!
    @IBOutlet var latitudeLabel: UILabel!
    @IBOutlet var longitudeLabel: UILabel!
    @IBOutlet var velocityLabel: UILabel!
    @IBOutlet var altitudeLabel: UILabel!
    @IBOutlet var headingLabel: UILabel!
    @IBOutlet var gpsTimeLabel: UILabel!
    @IBOutlet var gpsDateLabel: UILabel!
    @IBOutlet var stackView: UIStackView!
   
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.headingChange.addHandler(headingchange_handler)
            let _ = ch.positionChange.addHandler(positionchange_handler)
            let _ = ch.positionFixStateChange.addHandler(positionfix_handler)
            try ch.open()
            Timer.scheduledTimer(timeInterval: 0.1,
                                 target: self,
                                 selector: #selector(self.update(_:)),
                                 userInfo: nil,
                                 repeats: true)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @objc func update(_ timer:Timer){
        do{
            let time = try ch.getTime()
            let date = try ch.getDate()
            
            DispatchQueue.main.async {
                self.gpsDateLabel.text = String(format:"%02d/%02d/%02d", date.day, date.month, date.year)
                self.gpsTimeLabel.text = String(format:"%02d:%02d:%02d.%03d", time.hour,time.minute,time.second, time.millisecond)
            }
        }catch _ as PhidgetError {
            //ignore
        } catch {
            //ignore
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            DispatchQueue.main.async {
                self.stackView.isHidden = false
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func headingchange_handler(sender: GPS, data: (heading: Double, velocity: Double)){
        DispatchQueue.main.async {
            self.headingLabel.text = String(format:"%0.1fº", data.heading)
            self.velocityLabel.text = String(format:"%0.1f km/h", data.velocity)
        }
    }
    
    func positionchange_handler(sender: GPS, data: (latitude: Double, longitude: Double, altitude: Double)){
        DispatchQueue.main.async {
            self.latitudeLabel.text = String(format:"%0.3fº", data.latitude)
            self.longitudeLabel.text = String(format:"%0.3fº", data.longitude)
            self.altitudeLabel.text = String(format:"%0.3f m", data.altitude)
        }
    }
    
    func positionfix_handler(sender: GPS, positionFixState: Bool){
        DispatchQueue.main.async {
            if(positionFixState){
                self.fixAquiredLabel.text = "Fix Aquired!"
            }
            else{
                self.fixAquiredLabel.text = "Waiting for Fix..."
                self.latitudeLabel.text = ""
                self.longitudeLabel.text = ""
                self.altitudeLabel.text = ""
                self.velocityLabel.text = ""
                self.headingLabel.text = ""
                self.gpsTimeLabel.text = ""
                self.gpsDateLabel.text = ""
                
            }
        }
    }

    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }


}

